//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "CBWave.h"
#pragma resource "*.dcr"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//  TWaveObject Class constructor with inheristance
__fastcall TWaveObject::TWaveObject(void)
  :Classes::TPersistent()
{
   //create WaveAudio data buffer
   FWaveData = new TMemoryStream();

};
// TWaveObject Class Destructor
__fastcall TWaveObject::~TWaveObject(void)
/*  Classes::~TPersistent()*/
{
  FWaveData->Free();
}

// returns false if buffer is nil
bool __fastcall TWaveObject::GetEmpty(void)
{
  return (FWaveData == NULL);
};
// method to copy WaveAudio data when required
void __fastcall TWaveObject::Assign(Classes::TPersistent* Source)
{
  if ((Source == NULL) || (Source->ClassType()==this->ClassType()))
  {
    if (Source != NULL)
      FWaveData =((TWaveObject*)Source)->FWaveData;
    Changed(this);
    return;
  };
  Classes::TPersistent::Assign(Source);
};

// method to load WaveAudio data from a stream - used
// by the library and by the LoadFromFile method }
void __fastcall TWaveObject::LoadFromStream(Classes::TStream* Stream)
{
  FWaveData->SetSize(Stream->Size);
  Stream->ReadBuffer(FWaveData->Memory, Stream->Size);
  Changed(this);
};
// method to save WaveAudio data to a stream - used
// by the library and by the SaveToFile method }
void __fastcall TWaveObject::SaveToStream(Classes::TStream* Stream)
{
  Stream->WriteBuffer(FWaveData->Memory, FWaveData->Size);
};
// method to indicate data has changed - triggers
// the OnChange method }
void __fastcall TWaveObject::Changed(System::TObject* Sender)
{
  FModified = true;
  if (FOnChange!=NULL)
      FOnChange(this);
};
// method to allow "fake" data to be read and
// written by the library
void __fastcall TWaveObject::DefineProperties(Classes::TFiler* Filer)
{
  Filer->DefineBinaryProperty("Data", ReadData, WriteData, !Empty);
};

// method to set modified flag
void __fastcall TWaveObject::SetModified(bool Value)
{
  if (Value)
    Changed(this);
  else
    FModified = False;
};

// method to read data from *.wav file - calls
// LoadFromStream
void __fastcall TWaveObject::LoadFromFile(const System::AnsiString Filename)
{
  TStream *Stream;
  Stream  = new TFileStream(Filename, fmOpenRead);
  try
    {LoadFromStream(Stream);}
  __finally
    {Stream->Free();}
};

// method to write data to *.wav file - calls
// SaveToStream
void __fastcall TWaveObject::SaveToFile(const System::AnsiString Filename)
{
  TStream *Stream;
  Stream  = new TFileStream(Filename, fmCreate);
  try
    {SaveToStream(Stream);}
  __finally
    {Stream->Free();}
};

// method for library to read data from stream -
// calls LoadFromStream
void __fastcall TWaveObject::ReadData(Classes::TStream* Stream)
{
  LoadFromStream(Stream);
};

// method for library to write data to stream -
// calls SaveToStream
void __fastcall TWaveObject::WriteData(Classes::TStream* Stream)
{
  SaveToStream(Stream);
};
//=============================================================================
// TCBWavePlayer
//construct TCBWavePlayer
__fastcall TCBWavePlayer::TCBWavePlayer(Classes::TComponent* AOwner)
  :Classes::TComponent(AOwner)
{ 
  FBeforePlay = NULL;
  FAfterPlay = NULL;
  FEnabled = true;
  FOptions << wpoSync;
  Source =waveMemory;
  FWave = new TWaveObject();
  Source =waveMemory;
  ChangeCursor=true;
  WaveID  ="";
  PlayngCursor=crHourGlass;
}
// destructor
_fastcall TCBWavePlayer::~TCBWavePlayer(void)
{
  // must make this call or crash and burn 
  PlaySound(NULL,NULL,0);
  // free class instance 
  FreeWave();
}
//---------Generating the flags value
DWORD __fastcall TCBWavePlayer::Flag(void)
{
 DWORD Result=0;
   if(Source==waveMemory)   
     { Result=SND_MEMORY; };
   if(Source==waveFile)     
     { Result=SND_FILENAME; };
   if(Source==waveResource) 
     { Result=SND_RESOURCE; };
   if(Source==waveAlias)    
     { Result=SND_ALIAS; };
    if  (FOptions.Contains(wpoSync))
      Result |= SND_SYNC;
    if ( FOptions.Contains(wpoAsync)) 
      Result |= SND_ASYNC;
    if ( FOptions.Contains(wpoNoDefault)) 
      Result |= SND_NODEFAULT;
    if ( FOptions.Contains(wpoLoop)) 
      Result |= SND_LOOP;
    if ( FOptions.Contains(wpoNoStop)) 
      Result |= SND_NOSTOP;
    if ( FOptions.Contains(wpoNoWait)) 
      Result |= SND_NOWAIT;
    if ( FOptions.Contains(wpoPurge)) 
      Result |= SND_PURGE;
 return Result;
};
// SetBeforePlay method 
void __fastcall TCBWavePlayer::SetBeforePlay(Classes::TNotifyEvent Value)
{
  FBeforePlay = Value;
}
// SetAfterPlay method 
void __fastcall TCBWavePlayer::SetAfterPlay(Classes::TNotifyEvent Value)
{
   FAfterPlay = Value;
}
//method to copy WaveAudio data from property
//  editor, or programmatically
void __fastcall TCBWavePlayer::SetWave(TWaveObject* AWave)
{
  //copy data from source
  FWave->Assign(AWave);
}
//method to abort
void __fastcall TCBWavePlayer::Abort(void)
{
 TCursor OldCursor;
 try
   {
   if (ChangeCursor) 
    {
      OldCursor      = Screen->Cursor;
      Screen->Cursor = PlayngCursor;
    };
    PlaySound(NULL,NULL,0);
   }
   __finally
   {
     if (ChangeCursor)
        Screen->Cursor = OldCursor;
   };
}

//method to play WaveAudio data
void __fastcall TCBWavePlayer::Play(void)
{
 TCursor OldCursor;
  if (FBeforePlay!=NULL)  
      FBeforePlay(this); 
  // BeforePlay event 
  if (Enabled)
  try
    {
     if (!ChangeCursor) 
      {
        OldCursor      = Screen->Cursor;
        Screen->Cursor = PlayngCursor;
      };
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++      
// this generaly a importantest code piece
// plays memory resource wave
   if (Source==waveMemory)
   {
    PlaySound((char *)FWave->WaveData->Memory,NULL,Flag());
   }
// plays wave from module resoureces
   if (Source==waveResource)
   {
     PlaySound(WaveResource.c_str(),HInstance,Flag());
   }
// plays external file
   if (Source==waveFile)
   {
     PlaySound(WaveResource.c_str(),NULL,Flag());
   };
//plays Alias defined in win.ini
   if (Source==waveAlias)
   {
     PlaySound(WaveResource.c_str(),NULL,Flag());
   };

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    }
   __finally
    {
     if (!ChangeCursor)
        Screen->Cursor = OldCursor;
    };
  // AfterPlay event -- will execute before play has finished if
  //  Options.wpoAsync == true 
  if (FAfterPlay!=NULL) 
      FAfterPlay(this);

};

//metod for safely free Wave memory
void __fastcall TCBWavePlayer::FreeWave(void)
{
  if (FWave != NULL)
     { 
       delete FWave;
       FWave = NULL;
     };
}

//-- user supplied -----------------------------------------------------------
// tells CB3 that this is a dialog type property editor
Dsgnintf::TPropertyAttributes __fastcall TWaveEditor::GetAttributes(void)
{
  return (TPropertyAttributes() << paDialog) ;
};
// String value for the Editor
System::AnsiString __fastcall TWaveEditor::GetValue()
{
  return "TWaveObject";
}

// override of the edit method - this is the nuts
// and bolts of the property editor
void __fastcall TWaveEditor::Edit(void)
{ 

  WaveForm = new TWaveForm(NULL);
  OldWavePlayer = (TCBWavePlayer *)GetComponent(0);
  try
  {
    if (WaveForm->ShowModal()==mrOk)
       {
       if (OldWavePlayer->WaveSource==waveMemory) 
         {
          OldWavePlayer->Wave->LoadFromFile
            (WaveForm->FileListBox->FileName);
        }
          OldWavePlayer->WaveResource=WaveForm->FileListBox->FileName;
       }
  }
  __finally
  {
     delete WaveForm;
  }
};
//=============================================================================
// Wave Player Component editor
// tPlays wave if the component is double clicked
void __fastcall TWaveResourceEditor::Edit(void)
{
  ((TCBWavePlayer *)Component)->Play();
}

//=============================================================================
namespace Cbwave
{
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TCBWavePlayer *)
{
    new TCBWavePlayer(NULL);
}

//---------------------------------------------------------------------------
  void __fastcall Register(void)
  {
    TComponentClass classes[1] = {__classid(TCBWavePlayer)};
		
    RegisterComponents("Samples", classes, 0);
    RegisterPropertyEditor( __typeinfo(TWaveObject), NULL,"", __classid(TWaveEditor));
    RegisterComponentEditor(__classid(TCBWavePlayer), __classid(TWaveResourceEditor));
  }                             
}

